import { ParserItem, LexerItem, Searcher, Queue } from "./types";
/**
 [The "BSD licence"]
 Copyright (c) 2013 Tom Everett
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/**
   htmlDocument
    : scriptletOrSeaWs* XML? scriptletOrSeaWs* DTD? scriptletOrSeaWs* htmlElements*
    ;
 */
export declare class HtmlDocument implements ParserItem {
    scriptletOrSeaWs1: ScriptletOrSeaWs[];
    XML?: LexerItem<"XML">;
    scriptletOrSeaWs2: ScriptletOrSeaWs[];
    DTD?: LexerItem<"DTD">;
    scriptletOrSeaWs3: ScriptletOrSeaWs[];
    htmlElements: HtmlElements[];
    cache: {
        children: {
            invalid: boolean;
            value: HtmlElement[];
        };
        descendants: {
            invalid: boolean;
            value: HtmlElement[];
        };
        indexes: {
            invalid: boolean;
            value: Partial<Record<string, number>>;
        };
    };
    identifier: string;
    constructor();
    addComment: (text: string, after?: HtmlElement | number) => this;
    addConditionalComment: (text: string, after?: HtmlElement | number) => this;
    descendants: () => HtmlElement[];
    prevSibling: (element: HtmlElement) => HtmlElement | undefined;
    nextSibling: (element: HtmlElement) => HtmlElement | undefined;
    addChild: (child: HtmlElement, index?: number | undefined) => this;
    removeChild: (child: HtmlElement | number) => this;
    replaceChild: (child: HtmlElement | number, replacement: HtmlElement) => this;
    children: () => HtmlElement[];
    getIndex: (element: HtmlElement) => number;
    search: (searcher: Searcher) => void;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
}
/**
  scriptletOrSeaWs
    : SCRIPTLET
    | SEA_WS
    ;
 */
export declare class ScriptletOrSeaWs implements ParserItem {
    scriptlet?: LexerItem<"SCRIPTLET">;
    seaWs?: LexerItem<"SEA_WS">;
    search: (searcher: Searcher) => void;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
}
/**
   htmlElements
    : htmlMisc* htmlElement htmlMisc*
    ;
 */
export declare class HtmlElements implements ParserItem {
    htmlMisc1: HtmlMisc[];
    htmlElement?: HtmlElement;
    htmlMisc2: HtmlMisc[];
    parent: HtmlDocument;
    constructor(parent: HtmlDocument);
    search: (searcher: Searcher) => void;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    clone: () => HtmlElements;
}
/**
    htmlElement
        : TAG_OPEN TAG_NAME htmlAttribute*
        (TAG_CLOSE (htmlContent TAG_OPEN TAG_SLASH TAG_NAME TAG_CLOSE)? | TAG_SLASH_CLOSE)
        | SCRIPTLET
        | script
        | style
        ;
 */
export declare class HtmlElement implements ParserItem {
    tagOpen: LexerItem<"TAG_OPEN"> | undefined;
    tagWhiteSpace1: LexerItem<"TAG_WHITESPACE"> | undefined;
    tagName: LexerItem<"TAG_NAME"> | undefined;
    tagWhiteSpace2: LexerItem<"TAG_WHITESPACE"> | undefined;
    htmlAttributes: HtmlAttribute[];
    tagClose?: {
        close1?: {
            tagClose?: LexerItem<"TAG_CLOSE">;
            closingGroup?: {
                htmlContent?: HtmlContent;
                tagOpen?: LexerItem<"TAG_OPEN">;
                tagSlash?: LexerItem<"TAG_SLASH">;
                tagWhiteSpace1?: LexerItem<"TAG_WHITESPACE">;
                tagName?: LexerItem<"TAG_NAME">;
                tagWhiteSpace2?: LexerItem<"TAG_WHITESPACE">;
                tagClose?: LexerItem<"TAG_CLOSE">;
            };
        };
        close2?: {
            tagSlashClose?: LexerItem<"TAG_SLASH_CLOSE">;
        };
    };
    scriptlet?: LexerItem<"SCRIPTLET">;
    script?: Script;
    style?: Style;
    data: Partial<Record<string, string>>;
    parent: HtmlElement | HtmlDocument;
    identifier: string;
    cache: {
        attributes: {
            value: Partial<Record<string, string>>;
            invalid: boolean;
        };
        styles: {
            value: Partial<Record<string, string>>;
            invalid: boolean;
        };
        classNames: {
            value: Partial<Record<string, true>>;
            invalid: boolean;
        };
    };
    constructor(parent: HtmlElement | HtmlDocument);
    convertWithChildren: () => void;
    prevSibling: (element: HtmlElement) => HtmlElement | undefined;
    nextSibling: (element: HtmlElement) => HtmlElement | undefined;
    getStyles: () => Partial<Record<string, string>>;
    modifyStyle: (styleName: string, styleModifier: (value?: string) => string) => this;
    getTagName: () => string | undefined;
    content: () => HtmlContent | undefined;
    children: () => HtmlElement[];
    addText: (text: string) => this;
    emptyText: () => this;
    emptyComments: () => this;
    addComment: (text: string, after?: HtmlElement | number) => this;
    addConditionalComment: (text: string, after?: HtmlElement | number) => this;
    texts: () => string[];
    descendants: () => HtmlElement[];
    getIndex: (element: HtmlElement, indexOthers: boolean, filter?: ((element: HtmlElement) => boolean) | undefined) => number;
    addChild: (child: HtmlElement, index?: number | undefined) => this;
    removeChild: (child: HtmlElement | number) => this;
    replaceChild: (child: HtmlElement | number, replacement: HtmlElement) => this;
    getHtmlAttributes: () => HtmlAttribute[];
    addAttribute: (attributeName: string, attributeValue?: string) => this;
    removeAttribute: (attributeName: string) => this;
    modifyAttribute: (attributeName: string, modifier: (existingAttribute?: string) => undefined | string) => this;
    replaceAttribute: (attributeName: string, nextValue?: string) => this;
    attributes: () => Partial<Record<string, string>>;
    getClassNames: () => Partial<Record<string, true>>;
    noEndingTagNeeded: () => boolean;
    endTagConsumed: () => boolean;
    search: (searcher: Searcher) => void;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    clone: () => HtmlElement;
}
/**
   htmlContent
    : htmlChardata? ((htmlElement | CDATA | htmlComment) htmlChardata?)*
    ;
 */
export declare class HtmlContent implements ParserItem {
    htmlCharData?: HtmlChardata;
    content: Array<HtmlElement | LexerItem<"CDATA"> | HtmlComment | HtmlChardata>;
    cache: {
        children: {
            invalid: boolean;
            value: HtmlElement[];
        };
        indexes: {
            invalid: boolean;
            value: Partial<Record<string, number>>;
        };
    };
    identifier: string;
    parent: HtmlElement;
    constructor(parent: HtmlElement);
    children: () => HtmlElement[];
    descendants: () => HtmlElement[];
    emptyComments: () => void;
    emptyText: () => this;
    texts: () => string[];
    addText: (text: string, after?: HtmlElement) => this;
    addComment: (text: string, after?: HtmlElement | number) => this;
    addConditionalComment: (text: string, after?: HtmlElement | number) => this;
    addChild: (child: HtmlElement, index?: number | undefined) => this;
    removeChild: (child: HtmlElement | number) => this;
    replaceChild: (child: HtmlElement | number, replacement: HtmlElement) => this;
    getIndex: (element: HtmlElement, indexOthers: boolean, filter?: ((element: HtmlElement) => boolean) | undefined) => number;
    prevSibling: (element: HtmlElement) => HtmlElement | undefined;
    nextSibling: (element: HtmlElement) => HtmlElement | undefined;
    search: (searcher: Searcher) => void;
    process: (queue: Queue) => Queue;
    consumed: () => boolean;
    clone: () => HtmlContent;
}
/**
   htmlAttribute
    : TAG_NAME (TAG_EQUALS ATTVALUE_VALUE)?
    ;
 */
export declare class HtmlAttribute implements ParserItem {
    tagName?: LexerItem<"TAG_NAME">;
    attribute?: {
        tagEquals?: LexerItem<"TAG_EQUALS">;
        value?: LexerItem<"ATTVALUE_VALUE">;
        ws?: LexerItem<"TAG_WHITESPACE">;
    };
    search: (searcher: Searcher) => void;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    clone: () => HtmlAttribute;
}
/**
   htmlChardata
    : HTML_TEXT
    | SEA_WS
    ;
 */
export declare class HtmlChardata implements ParserItem {
    htmlText?: LexerItem<"HTML_TEXT">;
    seaWs?: LexerItem<"SEA_WS">;
    search: (searcher: Searcher) => void;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    clone: () => HtmlChardata;
}
/**
    htmlMisc
        : htmlComment
        | SEA_WS
        ;
 */
export declare class HtmlMisc implements ParserItem {
    htmlComment?: HtmlComment;
    seaWs?: LexerItem<"SEA_WS">;
    search: (searcher: Searcher) => void;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    clone: () => HtmlMisc;
}
/**
   htmlComment
    : HTML_COMMENT
    | HTML_CONDITIONAL_COMMENT
    ;
 */
export declare class HtmlComment implements ParserItem {
    htmlComment?: LexerItem<"HTML_COMMENT">;
    htmlConditionalComment?: LexerItem<"HTML_CONDITIONAL_COMMENT">;
    search: (searcher: Searcher) => void;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    clone: () => HtmlComment;
}
/**
  script
    : TAG_OPEN SCRIPT_OPEN htmlAttribute* ((TAG_CLOSE SCRIPT_BODY) | TAG_SLASH_CLOSE)
    ;
 */
export declare class Script implements ParserItem {
    scriptOpen?: LexerItem<"SCRIPT_OPEN">;
    scriptBody?: LexerItem<"SCRIPT_BODY">;
    tagClose?: LexerItem<"TAG_CLOSE">;
    tagSlashClose?: LexerItem<"TAG_SLASH_CLOSE">;
    attributes: HtmlAttribute[];
    search: (searcher: Searcher) => void;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    clone: () => Script;
}
/**
   style
    : TAG_OPEN STYLE_OPEN htmlAttribute* ((TAG_CLOSE STYLE_BODY) | TAG_SLASH_CLOSE)
    ;
 */
export declare class Style implements ParserItem {
    styleOpen?: LexerItem<"STYLE_OPEN">;
    styleBody?: LexerItem<"STYLE_BODY">;
    tagClose?: LexerItem<"TAG_CLOSE">;
    tagSlashClose?: LexerItem<"TAG_SLASH_CLOSE">;
    attributes: HtmlAttribute[];
    search: (searcher: Searcher) => void;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    clone: () => Style;
}
