/**
 * fragment TAG_NameChar:
    TAG_NameStartChar
    | '-'
    | '_'
    | '.'
    | DIGIT
    | '\u00B7'
    | '\u0300' ..'\u036F'
    | '\u203F' ..'\u2040'
;

fragment TAG_NameStartChar:
    [:a-zA-Z]
    | '\u2070' ..'\u218F'
    | '\u2C00' ..'\u2FEF'
    | '\u3001' ..'\uD7FF'
    | '\uF900' ..'\uFDCF'
    | '\uFDF0' ..'\uFFFD'
;
 */
export declare const htmlFragments: {
    readonly HEXADIGIT: () => RegExp;
    readonly DIGIT: () => RegExp;
    readonly TAG_NameChar: () => RegExp;
    readonly TAG_NameStartChar: () => RegExp;
    readonly ATTCHARS: () => RegExp;
    readonly ATTCHAR: () => RegExp;
    readonly HEXCHARS: () => RegExp;
    readonly DECCHARS: () => RegExp;
    readonly DOUBLE_QUOTE_STRING: () => RegExp;
    readonly SINGLE_QUOTE_STRING: () => RegExp;
};
export declare const htmlLexerAtoms: {
    readonly HTML_COMMENT: RegExp;
    readonly HTML_CONDITIONAL_COMMENT: RegExp;
    readonly XML: RegExp;
    readonly CDATA: RegExp;
    readonly DTD: RegExp;
    readonly SCRIPTLET: RegExp;
    readonly SEA_WS: RegExp;
    readonly SCRIPT_OPEN: {
        readonly value: RegExp;
        readonly pushMode: readonly ["SCRIPT", "TAG"];
    };
    readonly STYLE_OPEN: {
        readonly value: RegExp;
        readonly pushMode: readonly ["STYLE", "TAG"];
    };
    readonly TAG_OPEN: {
        readonly value: "<";
        readonly pushMode: readonly ["TAG"];
    };
    readonly HTML_TEXT: {
        readonly value: RegExp;
    };
    readonly TAG_CLOSE: {
        readonly value: ">";
        readonly popMode: true;
        readonly mode: "TAG";
    };
    readonly TAG_SLASH_CLOSE: {
        readonly value: "/>";
        readonly popMode: true;
        readonly mode: "TAG";
    };
    readonly TAG_SLASH: {
        readonly value: "/";
        readonly mode: "TAG";
    };
    readonly TAG_EQUALS: {
        readonly value: "=";
        readonly mode: "TAG";
        readonly pushMode: readonly ["ATTVALUE"];
    };
    readonly TAG_NAME: {
        readonly value: RegExp;
        readonly mode: "TAG";
    };
    readonly TAG_WHITESPACE: {
        readonly value: RegExp;
        readonly mode: "TAG";
    };
    readonly SCRIPT_BODY: {
        readonly value: (input: string, skip: number) => number | undefined;
        readonly popMode: true;
        readonly mode: "SCRIPT";
    };
    readonly STYLE_BODY: {
        readonly value: (input: string, skip: number) => number | undefined;
        readonly popMode: true;
        readonly mode: "STYLE";
    };
    readonly ATTVALUE_VALUE: {
        readonly value: RegExp;
        readonly popMode: true;
        readonly mode: "ATTVALUE";
    };
};
export declare const cssFragments: {
    readonly Newline: () => RegExp;
    readonly Hex: () => RegExp;
    readonly NewlineOrSpace: () => RegExp;
    readonly Unicode: () => RegExp;
    readonly Escape: () => RegExp;
    readonly Nonascii: () => RegExp;
    readonly Nmchar: () => RegExp;
    readonly Nmstart: () => RegExp;
    readonly Name: () => RegExp;
};
export declare const cssLexerAtoms: {
    readonly DashMatch: "|=";
    readonly Includes: "~=";
    readonly PrefixMatch: "^=";
    readonly SuffixMatch: "$=";
    readonly SubstringMatch: "*=";
    readonly Plus: "+";
    readonly Minus: "-";
    readonly Greater: ">";
    readonly Comma: ",";
    readonly Tilde: "~";
    readonly Dot: ".";
    readonly PseudoNot: ":not(";
    readonly PseudoGeneral: RegExp;
    readonly Namespace: "|";
    readonly Universal: "*";
    readonly Number: RegExp;
    readonly Equals: "=";
    readonly SquareBracket: "[";
    readonly SquareBracketEnd: "]";
    readonly Space: RegExp;
    readonly String_: RegExp;
    readonly Of: "of";
    readonly Odd: "odd";
    readonly Even: "even";
    readonly Function_: RegExp;
    readonly Hash: RegExp;
    readonly Ident: RegExp;
    readonly BackBrace: ")";
};
export type LexerType = keyof typeof htmlLexerAtoms | keyof typeof cssLexerAtoms | "EOF";
export interface Lexer {
    value: string | RegExp | ((input: string, skip: number) => (number | undefined));
    popMode?: boolean;
    mode?: "SCRIPT" | "STYLE" | "TAG" | "ATTVALUE";
    pushMode?: ReadonlyArray<"SCRIPT" | "STYLE" | "TAG" | "ATTVALUE">;
}
export declare const normalizeHtmlLexer: (item: keyof typeof htmlLexerAtoms) => {
    readonly value: "<";
    readonly pushMode: readonly ["TAG"];
} | {
    readonly value: RegExp;
} | {
    readonly value: ">";
    readonly popMode: true;
    readonly mode: "TAG";
} | {
    readonly value: "/>";
    readonly popMode: true;
    readonly mode: "TAG";
} | {
    readonly value: "/";
    readonly mode: "TAG";
} | {
    readonly value: "=";
    readonly mode: "TAG";
    readonly pushMode: readonly ["ATTVALUE"];
} | {
    readonly value: (input: string, skip: number) => number | undefined;
    readonly popMode: true;
    readonly mode: "SCRIPT";
} | {
    readonly value: (input: string, skip: number) => number | undefined;
    readonly popMode: true;
    readonly mode: "STYLE";
};
