import { HtmlDocument, HtmlElement } from "./html";
interface TypeCombiner {
    attr(name: string): string | undefined;
    attr(attributes: Record<string, string>): QueryInstance;
    attr(attributeName: string, value: string | ((index: number, value: string) => string)): QueryInstance;
    css(propertyName: string): string | undefined;
    css(css: Record<string, string>): QueryInstance;
    css(propertyName: string, value: string | ((index: number, value: string) => string)): QueryInstance;
    data(key: string): string | undefined;
    data(object: Record<string, string>): QueryInstance;
    data(): Record<string, string>;
    data(key: string, value: string): QueryInstance;
    html(): string;
    html(value: string): QueryInstance;
    html(setter: (index: number, value: string) => string): QueryInstance;
    tagName(): string;
    tagName(value: string): QueryInstance;
    tagName(setter: (index: number, tagName: string) => string): QueryInstance;
    text(): string;
    text(value: string): QueryInstance;
    text(setter: (index: number, value: string) => string): QueryInstance;
    val(): string;
    val(value: string | number | boolean): QueryInstance;
    val(setter: (index: number, value: string) => string | number | boolean): QueryInstance;
    wrap(parent: QueryInstance): QueryInstance;
    wrap(parent: (index: number, child: QueryInstance) => QueryInstance | string): QueryInstance;
    wrap(parent: string): QueryInstance;
    wrapAll(parent: string): QueryInstance;
    wrapAll(parent: QueryInstance): QueryInstance;
    wrapAll(parent: () => QueryInstance | string): QueryInstance;
    height(): string;
    height(value: string): QueryInstance;
    width(): string;
    width(value: string): QueryInstance;
}
type ValueType = TypeCombiner["val"];
type TagNameType = TypeCombiner["tagName"];
type TextType = TypeCombiner["text"];
type HtmlType = TypeCombiner["html"];
type DataType = TypeCombiner["data"];
type CssType = TypeCombiner["css"];
type AttrType = TypeCombiner["attr"];
type WrapType = TypeCombiner["wrap"];
type WrapAllType = TypeCombiner["wrapAll"];
declare class QueryInstance implements Record<number, QueryInstance> {
    private document;
    private virtualDoms;
    private matched;
    private previous;
    private compress;
    private getUntil;
    private addElementsToMatcher;
    private getLowestChild;
    private getParentAssigner;
    private filterBySelector;
    constructor(document: HtmlDocument, matched: HtmlElement[], virtualDoms: HtmlDocument[], previous: QueryInstance | {
        compress: boolean;
    });
    [x: number]: QueryInstance;
    [Symbol.iterator](): Generator<QueryInstance, void, unknown>;
    find: (queryInput: string | QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    add: (toAdd: QueryInstance | string, context?: QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    addBack: (selector?: string, namespaces?: Record<string, string>) => QueryInstance;
    addClass: (classNames: string | ((index: number, classNames: string) => string | string[])) => this;
    after: (firstArg: string | ((index: number, html: string) => string), ...content: string[]) => this;
    append: (firstArg: string | ((index: number, html: string) => string), ...content: string[]) => this;
    appendTo: (query: QueryInstance | string, namespaces?: Record<string, string>) => this;
    attr: AttrType;
    before: (firstArg: string | ((index: number, html: string) => string), ...content: string[]) => this;
    blur: (handler: string | ((event: Event) => void)) => this;
    change: (handler: string | ((event: Event) => void)) => this;
    children: () => QueryInstance;
    clone: () => QueryInstance;
    click: (handler: string | ((event: Event) => void)) => this;
    closest: (selector: string, container?: QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    contents: () => string;
    contextmenu: (handler: string | ((event: Event) => void)) => this;
    css: CssType;
    data: DataType;
    dblclick: (handler: string | ((event: Event) => void)) => this;
    empty: () => this;
    end: () => QueryInstance;
    eq: (index: number) => QueryInstance;
    filter: (filter: string | QueryInstance | ((index: number, instance: QueryInstance) => any), namespaces?: Record<string, string>) => QueryInstance;
    each: (callback: (index: number, query: QueryInstance) => void) => this;
    even: () => QueryInstance;
    first: () => QueryInstance;
    focus: (handler: string | ((event: Event) => void)) => this;
    focusin: (handler: string | ((event: Event) => void)) => this;
    focusout: (handler: string | ((event: Event) => void)) => this;
    has: (selector: string | QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    hasClass: (className: string) => boolean;
    hide: () => this;
    hover: (handler: string | ((event: Event) => void)) => this;
    html: HtmlType;
    index: (selector?: string | QueryInstance, namespaces?: Record<string, string>) => number;
    insertBefore: (selector: string | QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    insertAfter: (selector: string | QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    keypress: (handler: string | ((event: Event) => void)) => this;
    keydown: (handler: string | ((event: Event) => void)) => this;
    keyup: (handler: string | ((event: Event) => void)) => this;
    is: (selector: string | QueryInstance, namespaces?: Record<string, string>) => boolean;
    last: () => QueryInstance;
    get length(): number;
    map: <T>(iterator: (index: number, item: QueryInstance) => T) => T[];
    mousedown: (handler: string | ((event: Event) => void)) => this;
    mouseenter: (handler: string | ((event: Event) => void)) => this;
    mouseleave: (handler: string | ((event: Event) => void)) => this;
    mousemove: (handler: string | ((event: Event) => void)) => this;
    mouseout: (handler: string | ((event: Event) => void)) => this;
    mouseover: (handler: string | ((event: Event) => void)) => this;
    mouseup: (handler: string | ((event: Event) => void)) => this;
    next: (selector?: string | QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    nextAll: (selector?: string | QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    nextUntil: (selector: string | QueryInstance, filter?: string | QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    not: (selector: string | QueryInstance | ((index: number, element: QueryInstance) => boolean), namespaces?: Record<string, string>) => QueryInstance;
    odd: () => QueryInstance;
    off: (eventTypes: string) => this;
    on: (eventTypes: string, eventHandler: string | ((event: Event) => void)) => this;
    once: (eventTypes: string, eventHandler: string | ((event: Event) => void)) => this;
    parent: (selector?: string | QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    parents: (selector?: string | QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    parentsUntil: (selector: string | QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    prepend: (firstArg: string | ((index: number, html: string) => string), ...content: string[]) => this;
    prependTo: (query: string | QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    prev: (selector?: string | QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    prevAll: (selector?: string | QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    prevUntil: (selector: string | QueryInstance, filter?: string | QueryInstance, namespaces?: Record<string, string>) => QueryInstance;
    print: (ignoreWhitespace?: boolean) => string;
    /**
     * Using this instead of prop()
     */
    tagName: TagNameType;
    pushStack: (instance: QueryInstance) => QueryInstance;
    ready: (handler: string | ((event: Event) => void)) => this;
    reduce: <TAccumulator>(reducer: (accumulator: TAccumulator, value: QueryInstance, index: number, self: QueryInstance) => TAccumulator, initialValue: TAccumulator) => TAccumulator;
    remove: (selector?: string, namespaces?: Record<string, string>) => QueryInstance;
    removeAttr: (attributeNames: string) => this;
    removeClass: (classNames: string | ((index: number, classNames: string) => string | string[])) => this;
    removeData: (data: string | string[]) => this;
    replaceAll: (replacement: string | QueryInstance | string[] | QueryInstance[], namespaces?: Record<string, string>) => this;
    replaceWith: (content: string | string[] | (() => string | string[] | QueryInstance | QueryInstance[]) | QueryInstance | QueryInstance[]) => QueryInstance;
    select: (handler: string | ((event: Event) => void)) => this;
    siblings: (selector?: string, namespaces?: Record<string, string>) => QueryInstance;
    slice: (from: number, to?: number) => QueryInstance;
    submit: (handler: string | ((event: Event) => void)) => this;
    text: TextType;
    toggleClass: (classNames: string | ((index: number, classNames: string) => string | string[]), state?: boolean) => this;
    uniqueSort: () => QueryInstance;
    unload: (handler: string | ((event: Event) => void)) => this;
    unwrap: (selector?: string | QueryInstance, namespaces?: Record<string, string>) => this;
    val: ValueType;
    wrap: WrapType;
    wrapAll: WrapAllType;
    wrapInner: WrapType;
}
export declare const Query: (htmlInput: string, compress?: boolean) => (queryInput?: string | ((event: Event) => void) | undefined, namespaces?: Record<string, string>) => QueryInstance;
export {};
