import { HtmlElement } from "./html";
import { LexerItem, ParserItem, Queue, Searcher } from "./types";
export interface Matcher extends ParserItem {
    match: (htmlElements: HtmlElement[], allHtmlElements: HtmlElement[], namespaces: Record<string, string>) => HtmlElement[];
}
/**
 * selectorGroup
    : selector ( Comma ws selector )*
    ;
 */
export declare class SelectorGroup implements Matcher {
    selector?: Selector;
    selectors: Array<LexerItem<"Comma"> | Ws | Selector>;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    match: (htmlElements: HtmlElement[], allHtmlElements: HtmlElement[], namespaces: Record<string, string>) => HtmlElement[];
}
/**
 * selector
    : simpleSelectorSequence ( combinator simpleSelectorSequence )*
    ;
 */
export declare class Selector implements Matcher {
    simpleSelectorSequence?: SimpleSelectorSequence;
    sequences: Array<Combinator | SimpleSelectorSequence>;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    match: (htmlElements: HtmlElement[], allHtmlElements: HtmlElement[], namespaces: Record<string, string>) => HtmlElement[];
}
/**
    combinator
        : ws Plus ws
        | ws Greater ws
        | ws Tilde ws
        | ws Space
        ;
 */
export declare class Combinator implements Matcher {
    ws1?: Ws;
    plus?: LexerItem<"Plus">;
    greater?: LexerItem<"Greater">;
    tilde?: LexerItem<"Tilde">;
    space?: LexerItem<"Space">;
    ws2?: Ws;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    match: (htmlElements: HtmlElement[], _: HtmlElement[], __: Record<string, string>) => HtmlElement[];
}
/**
    simpleSelectorSequence
        : ( typeSelector | universal ) ( Hash | className | attrib | pseudo | negation )*
        | ( Hash | className | attrib | pseudo | negation )+
        ;
 */
export declare class SimpleSelectorSequence implements Matcher {
    typeSelector?: TypeSelector;
    universal?: Universal;
    modifiers: Array<LexerItem<"Hash"> | ClassName | Attrib | Pseudo | Negation>;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    match: (htmlElements: HtmlElement[], allHtmlElements: HtmlElement[], namespaces: Record<string, string>) => HtmlElement[];
}
/**
 * pseudo
    : PseudoGeneral ( ident | functionalPseudo )
    ;
 */
export declare class Pseudo implements Matcher {
    pseudoGeneral?: LexerItem<"PseudoGeneral">;
    ident?: LexerItem<"Ident">;
    functionalPseudo?: FunctionalPseudo;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    match: (htmlElements: HtmlElement[], allHtmlElements: HtmlElement[], namespaces: Record<string, string>) => HtmlElement[];
}
/**
   functionalPseudo
    : Function_ ws expression ')'
    ;
 */
export declare class FunctionalPseudo implements Matcher {
    funct?: LexerItem<"Function_">;
    ws?: Ws;
    expression?: Expression;
    backBrace?: LexerItem<"BackBrace">;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    match: (htmlElements: HtmlElement[], allHtmlElements: HtmlElement[], namespaces: Record<string, string>) => HtmlElement[];
}
/**
   typeNamespacePrefix
    : ( ident | '*' )? '|'
    ;
 */
export declare class TypeNamespacePrefix implements Matcher {
    ident?: LexerItem<"Ident">;
    namespace?: LexerItem<"Namespace">;
    universal?: LexerItem<"Universal">;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    match: (htmlElements: HtmlElement[], _: HtmlElement[], namespaces: Record<string, string>) => HtmlElement[];
}
export declare class TypeSelector implements Matcher {
    typeNamespacePrefix?: TypeNamespacePrefix;
    elementName?: ElementName;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    match: (htmlElements: HtmlElement[], allHtmlElements: HtmlElement[], namespaces: Record<string, string>) => HtmlElement[];
}
export declare class ElementName implements Matcher {
    ident?: LexerItem<"Ident">;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    match: (htmlElements: HtmlElement[], _: HtmlElement[], __: Record<string, string>) => HtmlElement[];
}
/**
   universal
    : typeNamespacePrefix? '*'
    ;
 */
export declare class Universal implements Matcher {
    typeNamespacePrefix?: TypeNamespacePrefix;
    universal?: LexerItem<"Universal">;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    match: (htmlElements: HtmlElement[], allHtmlElements: HtmlElement[], namespaces: Record<string, string>) => HtmlElement[];
}
/**
   className
    : '.' ident
    ;
 */
export declare class ClassName implements Matcher {
    dot?: LexerItem<"Dot">;
    ident?: LexerItem<"Ident">;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    match: (htmlElements: HtmlElement[], _: HtmlElement[], __: Record<string, string>) => HtmlElement[];
}
/**
    attrib
    : '[' ws typeNamespacePrefix? ident ws ( ( PrefixMatch | SuffixMatch | SubstringMatch | '=' | Includes | DashMatch ) ws ( ident | String_ ) ws )? ']'
    ;
 */
export declare class Attrib implements Matcher {
    squareBracket?: LexerItem<"SquareBracket">;
    ws1?: Ws;
    typeNamespacePrefix?: TypeNamespacePrefix;
    ident1?: LexerItem<"Ident">;
    ws2?: Ws;
    prefixMatch?: LexerItem<"PrefixMatch">;
    suffixMatch?: LexerItem<"SuffixMatch">;
    substringMatch?: LexerItem<"SubstringMatch">;
    equals?: LexerItem<"Equals">;
    includes?: LexerItem<"Includes">;
    dashMatch?: LexerItem<"DashMatch">;
    ws3?: Ws;
    ident2?: LexerItem<"Ident">;
    stringIdent?: LexerItem<"String_">;
    numIdent?: LexerItem<"Number">;
    ws4?: Ws;
    squareBracketEnd?: LexerItem<"SquareBracketEnd">;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    match: (htmlElements: HtmlElement[], _: HtmlElement[], __: Record<string, string>) => HtmlElement[];
}
/**
    negation
        : PseudoNot ws negationArg ws ')'
        ;
 */
export declare class Negation implements Matcher {
    pseudoNot?: LexerItem<"PseudoNot">;
    ws1?: Ws;
    negationArg?: NegationArg;
    ws2?: Ws;
    backBrace?: LexerItem<"BackBrace">;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    match: (htmlElements: HtmlElement[], allHtmlElements: HtmlElement[], namespaces: Record<string, string>) => HtmlElement[];
}
/**
    negationArg
        : typeSelector
        | universal
        | Hash
        | className
        | attrib
        | pseudo
        ;
 */
export declare class NegationArg implements Matcher {
    typeSelector?: TypeSelector;
    universal?: Universal;
    hash?: LexerItem<"Hash">;
    className?: ClassName;
    attrib?: Attrib;
    pseudo?: Pseudo;
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    match: (htmlElements: HtmlElement[], allHtmlElements: HtmlElement[], namespaces: Record<string, string>) => HtmlElement[];
}
/**
    ws
        : Space*
        ;
*/
export declare class Ws implements ParserItem {
    spaces: LexerItem<"Space">[];
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
}
/**
   expression
    :
        Even
        | Odd
        | Combinator? selectorGroup
        | Minus? Number (ident ws (( Plus | Minus ) ws Number)?)? ws ( Of ws selectorGroup)? )?
        | ident
        
    ;
 */
export declare class Expression implements Matcher {
    even?: LexerItem<"Even">;
    odd?: LexerItem<"Odd">;
    combinator?: Combinator;
    selectorGroup1?: SelectorGroup;
    minus1?: LexerItem<"Minus">;
    number1?: LexerItem<"Number">;
    ident1?: LexerItem<"Ident">;
    ws2?: Ws;
    plus?: LexerItem<"Plus">;
    minus2?: LexerItem<"Minus">;
    ws3?: Ws;
    number2?: LexerItem<"Number">;
    ws4?: Ws;
    of?: LexerItem<"Of">;
    ws5?: Ws;
    selectorGroup2?: SelectorGroup;
    functionalPseudo: FunctionalPseudo;
    constructor(functionalPseudo: FunctionalPseudo);
    consumed: () => boolean;
    process: (queue: Queue) => Queue;
    search: (searcher: Searcher) => void;
    getIndex: (element: HtmlElement, selected: HtmlElement[]) => number;
    match: (htmlElements: HtmlElement[], allHtmlElements: HtmlElement[], namespaces: Record<string, string>) => HtmlElement[];
}
