import { LexerItem, ParserItem } from "./types";
export declare const getItems: <T extends ParserItem>(document: ParserItem, validator: (item: ParserItem | LexerItem<any>) => item is T) => T[];
export declare const parserItemToString: (document?: ParserItem) => string;
export declare const desanitizeAttribute: (attributeValue: string) => string;
export declare const sanitizeAttribute: (attribute: string | undefined) => string | undefined;
/**
    [attr]
        Represents elements with an attribute name of attr.
    [attr=value]
        Represents elements with an attribute name of attr whose value is exactly value.
    [attr~=value]
        Represents elements with an attribute name of attr whose value is a whitespace-separated list of words, one of which is exactly value.
    [attr|=value]
        Represents elements with an attribute name of attr whose value can be exactly value or can begin with value immediately followed by a hyphen, - (U+002D). It is often used for language subcode matches.
    [attr^=value]
        Represents elements with an attribute name of attr whose value is prefixed (preceded) by value.
    [attr$=value]
        Represents elements with an attribute name of attr whose value is suffixed (followed) by value.
    [attr*=value]
        Represents elements with an attribute name of attr whose value contains at least one occurrence of value within the string.
 */
export declare const matchAttribute: <T extends string | boolean>(attributes: Partial<Record<string, string>>, attributeName: string, attributeValue: T, matchType: T extends boolean ? "[attr]" | "not" : "[attr]" | "not" | "[attr=value]" | "[attr~=value]" | "[attr|=value]" | "[attr^=value]" | "[attr$=value]" | "[attr*=value]") => boolean;
/**
    week 	yyyy-W## 	<input type="week" min="2019-W23" step="">
    time 	hh:mm 	<input type="time" min="09:00" step="900">
    
    month 	yyyy-mm 	<input type="month" min="2019-12" step="12">
    date 	yyyy-mm-dd 	<input type="date" min="2019-12-25" step="1">
    datetime-local 	yyyy-mm-ddThh:mm 	<input type="datetime-local" min="2019-12-25T19:30">
    
    number 	<number> 	<input type="number" min="0" step="5" max="100">
    range 	<number> 	<input type="range" min="60" step="5" max="100">
 */
export declare const rangeComparator: (attributes: Partial<Record<string, string>>) => boolean;
/**
 * pattern, min, max, required, step, minlength, maxlength
 */
export declare const inputValidation: (attributes: Partial<Record<string, string>>, getAttributes: () => Partial<Record<string, string>>[]) => boolean;
export declare const flatten: <T>(multi: T[][]) => T[];
export declare const uniqueId: (prefix: string) => string;
